/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <time.h>
#include <contacts.h>
#include "contacts_app.h"
#include "data/data_contact_details.h"

/**
 * @brief Toggles is_favourite value for person record.
 * @param[in] person_id ID of a person.
 * @return This function returns 'true' if the favorite flag was toggled successfully,
 * otherwise 'false' is returned.
 */
bool data_contact_details_toggle_favourites(int person_id)
{
	bool ret;
	bool is_favourite = false;
	contacts_record_h rec_h = NULL;

	if (!data_common_get_record(_contacts_person._uri, person_id, &rec_h))
		return false;

	if (!data_common_get_bool(rec_h, _contacts_person.is_favorite, &is_favourite) ||
		!data_common_set_bool(rec_h, _contacts_person.is_favorite, !is_favourite)) {
		contacts_record_destroy(rec_h, true);
		return false;
	}

	ret = data_common_update_record(rec_h);
	contacts_record_destroy(rec_h, true);

	return ret;
}

/**
 * @brief Gets the 'favorite' state of the contact.
 * @param[in] person_id ID of a person.
 * @return This function returns 'true' if the favorite flag was obtained successfully,
 * otherwise 'false' is returned.
 */
bool data_contact_details_is_favourite(int person_id, bool *is_favourite)
{
	bool ret;
	contacts_record_h rec_h = NULL;

	if (!is_favourite) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_details_is_favourite() failed. Invalid input argument.");
		return false;
	}

	if (!data_common_get_record(_contacts_person._uri, person_id, &rec_h))
		return false;

	ret = data_common_get_bool(rec_h, _contacts_person.is_favorite, is_favourite);
	contacts_record_destroy(rec_h, true);

	return ret;
}

/**
 * @brief Retrieves the contact's field value.
 * @param[in] field_type Type of field.
 * @param[in] contact_id ID of contact.
 * @param[out] value The contact's field value. This value must be freed when it is not used anymore.
 * @return This function returns 'true' if the contact's field value was obtained successfully,
 * otherwise 'false' is returned.
 */
bool data_contact_details_get_value(sc_common_contact_fields field_type, int contact_id, char **value)
{
	bool ret = false;
	int int_date = 0;
	struct tm date;
	unsigned parent_property_id = 0;
	unsigned child_property_id = 0;
	contacts_record_h rec_h;
	contacts_record_h child_h;

	if (field_type >= CF_MAX || field_type < CF_PHONE_NUMBER) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_details_get_value() failed. Invalid input argument.");
		return false;
	}

	data_common_get_contact_properties(field_type, &parent_property_id, NULL, &child_property_id);

	if (!data_common_get_record(_contacts_contact._uri, contact_id, &rec_h))
		return false;

	if (!data_common_get_record_child(rec_h, parent_property_id, &child_h)) {
		contacts_record_destroy(rec_h, false);
		return false;
	}

	if (field_type == CF_BIRTHDAY) {
		if (data_common_get_int(child_h, child_property_id, &int_date)) {
			date = data_common_int_to_date(int_date);
			*value = data_common_date_to_str(&date);
			ret = true;
		}
	} else {
		ret = data_common_get_str(child_h, child_property_id, value);
	}

	contacts_record_destroy(rec_h, false);

	return ret;
}
