/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include "contacts_app.h"
#include "data/data_contact_edit.h"
#include "data/data_common.h"

/**
 * @brief Creates new contact data structure.
 * @return This function returns data structure if it was successfully created,
 * otherwise NULL is returned.
 */
sc_ec_contact_edit *data_contact_edit_new(void)
{
	sc_ec_contact_edit *data = (sc_ec_contact_edit *)malloc(sizeof(sc_ec_contact_edit));
	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "malloc() failed.");
		return NULL;
	}

	if (!data_common_create_record(_contacts_contact._uri, &data->contact_record)) {
		free(data);
		return NULL;
	}

	data->is_new_contact = true;

	return data;
}

/**
 * @brief Gets contact data structure.
 * @param[in] contact_id ID of contact to edit.
 * @return This function returns data structure if it was successfully created,
 * otherwise NULL is returned.
 */
sc_ec_contact_edit *data_contact_edit_get(int contact_id)
{
	sc_ec_contact_edit *data = (sc_ec_contact_edit *)malloc(sizeof(sc_ec_contact_edit));
	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "malloc() failed.");
		return NULL;
	}

	if (!data_common_get_record(_contacts_contact._uri, contact_id, &data->contact_record)) {
		free(data);
		return NULL;
	}

	data->is_new_contact = false;

	return data;
}

/**
 * @brief Destroys the contact data.
 * @param[in] data Structure to destroy.
 */
void data_contact_edit_destroy(sc_ec_contact_edit *data)
{
	if (!data) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_destroy_record() failed. Invalid input argument.");
		return;
	}

	if (data->contact_record)
		contacts_record_destroy(data->contact_record, true);

	free(data);
}

/**
 * @brief Saves the contact data to the database.
 * @param[in] data Contact data to store.
 * @return Function returns the identifier of created/updated contact if successful,
 * otherwise 0 is returned.
 */
int data_contact_edit_save(sc_ec_contact_edit *data)
{
	int rec_id = 0;

	if (!data || !data->contact_record) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_save() failed. Invalid input argument.");
		return 0;
	}

	if (data->is_new_contact)
		data_common_insert_record(data->contact_record, &rec_id);
	else if (data_common_update_record(data->contact_record))
		data_common_get_int(data->contact_record, _contacts_contact.id, &rec_id);

	return rec_id;
}

/**
 * @brief Gets the string value of contact's record field.
 * @param[in] field_type Type of field: name, number, etc.
 * @param[in] rec_h Contact's record handle.
 * @param[out] value Obtained string value.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_contact_edit_get_str_value(sc_common_contact_fields field_type, contacts_record_h rec_h, char **value)
{
	unsigned parent_property_id = 0;
	unsigned child_property_id = 0;
	contacts_record_h child_h;

	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME || !rec_h || !value) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_get_str_value() failed. Invalid input argument.");
		return false;
	}

	data_common_get_contact_properties(field_type, &parent_property_id, NULL, &child_property_id);

	if (!data_common_get_record_child(rec_h, parent_property_id, &child_h))
		return false;

	return data_common_get_str(child_h, child_property_id, value);
}

/**
 * @brief Gets the integer value of contact's record field.
 * @param[in] field_type Type of field: name, number, etc.
 * @param[in] rec_h Contact's record handle.
 * @param[out] value Obtained integer value.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_contact_edit_get_int_value(sc_common_contact_fields field_type, contacts_record_h rec_h, int *value)
{
	unsigned parent_property_id = 0;
	unsigned child_property_id = 0;
	contacts_record_h child_h;

	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME || !rec_h || !value) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_get_int_value() failed. Invalid input argument.");
		return false;
	}

	data_common_get_contact_properties(field_type, &parent_property_id, NULL, &child_property_id);

	if (!data_common_get_record_child(rec_h, parent_property_id, &child_h))
		return false;

	return data_common_get_int(child_h, child_property_id, value);
}

/**
 * @brief Sets the string value to the contact's record field.
 * @param[in] field_type Type of field: name, number, etc.
 * @param[in] rec_h Contact's record handle.
 * @param[in] value The string value to be set.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_contact_edit_set_str_value(sc_common_contact_fields field_type, contacts_record_h rec_h, const char *value)
{
	unsigned parent_property_id = 0;
	unsigned child_property_id = 0;
	const char *child_uri = NULL;
	contacts_record_h child_h;

	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME || !rec_h || !value) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_set_str_value() failed. Invalid input argument.");
		return false;
	}

	data_common_get_contact_properties(field_type, &parent_property_id, &child_uri, &child_property_id);

	if (!data_common_get_record_child(rec_h, parent_property_id, &child_h))
		if (!data_common_add_record_child(rec_h, child_uri, parent_property_id, &child_h))
			return false;

	return data_common_set_str(child_h, child_property_id, value);
}

/**
 * @brief Sets the integer value to the contact's record field.
 * @param[in] field_type Type of field: name, number, etc.
 * @param[in] rec_h Contact's record handle.
 * @param[in] value The integer value to be set.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_contact_edit_set_int_value(sc_common_contact_fields field_type, contacts_record_h rec_h, int value)
{
	unsigned parent_property_id = 0;
	unsigned child_property_id = 0;
	const char *child_uri = NULL;
	contacts_record_h child_h;

	if (field_type >= CF_MAX || field_type < CF_FIRST_NAME || !rec_h) {
		dlog_print(DLOG_ERROR, LOG_TAG, "data_contact_edit_set_str_value() failed. Invalid input argument.");
		return false;
	}

	data_common_get_contact_properties(field_type, &parent_property_id, &child_uri, &child_property_id);

	if (!data_common_get_record_child(rec_h, parent_property_id, &child_h))
		if (!data_common_add_record_child(rec_h, child_uri, parent_property_id, &child_h))
			return false;

	return data_common_set_int(child_h, child_property_id, value);
}

/**
 * @brief Checks whether first or last name is set in the provided record.
 * @param[in] rec_h Contact's record handle.
 * @return This function returns 'true' if the function succeeds,
 * otherwise 'false' is returned.
 */
bool data_contact_edit_check_contact_name(contacts_record_h rec_h)
{
	bool first;
	bool last;
	char *name = NULL;
	contacts_record_h child_h;

	if (!data_common_get_record_child(rec_h, _contacts_contact.name, &child_h))
		return false;

	first = (data_common_get_str(child_h, _contacts_name.first, &name) && name);
	if (first)
		free(name);

	name = NULL;
	last = (data_common_get_str(child_h, _contacts_name.last, &name) && name);
	if (last)
		free(name);

	return first || last;
}

/**
 * @brief Function obtains the date from a record.
 * @param[in] record Record where a date is stored.
 * @return This function returns tm structure for date representation.
 * On error, the tm structure is zeroed.
 */
struct tm data_contact_edit_get_date(contacts_record_h rec_h)
{
	struct tm date = {0};
	int int_date = 0;

	if (data_contact_edit_get_int_value(CF_BIRTHDAY, rec_h, &int_date))
		date = data_common_int_to_date(int_date);
	else {
		time_t now = time(NULL);
		localtime_r(&now, &date);
	}

	return date;
}
