/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <contacts.h>
#include <contacts_types.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_contact_details.h"
#include "view/view_contact_edit.h"
#include "view/view_common.h"
#include "data/data_contact_details.h"
#include "data/data_common.h"

#define CAPTION_BT_FAVOURITE "Fav."
#define CAPTION_BT_FAVOURITE_PRESSED "<color=#0F0>Fav.</>"

static struct view_info {
	Evas_Object *navi;
	Evas_Object *genlist;
	Evas_Object *popup;
	Elm_Object_Item *navi_item;
	int person_id;
	int contact_id;
} s_info = {
	.navi = NULL,
	.genlist = NULL,
	.popup = NULL,
	.navi_item = NULL,
	.person_id = 0,
	.contact_id = 0,
};

static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _delete_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _popup_ok_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _favourite_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _update_details_cb(const char *view_uri, void *user_data);
static char *_get_item_text_cb(void *data, Evas_Object *obj, const char *part);
static void _populate_list(void);
static bool _get_contact_id(int person_id, int *contact_id);
static bool _get_contact_name(int person_id, char **name);

/**
 * @brief Creates contact's detailed view.
 * @param[in] parent Parent object.
 * @param[in] person_id ID of person.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_contact_details_create(Evas_Object *parent, int person_id)
{
	int ret;
	Evas_Object *toolbar;
	bool is_favourite = false;
	char *person_name = NULL;
	const char *fav_bt_caption = NULL;

	if (!parent || person_id <= 0) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for contact details view.");
		return EINA_FALSE;
	}

	if (!_get_contact_id(person_id, &s_info.contact_id) ||
		!_get_contact_name(person_id, &person_name))
		return EINA_FALSE;

	s_info.navi = parent;
	s_info.person_id = person_id;
	s_info.genlist = view_create_genlist(s_info.navi);
	if (!s_info.genlist)
		return EINA_FALSE;

	_populate_list();

	s_info.navi_item = elm_naviframe_item_push(s_info.navi, person_name, NULL, NULL, s_info.genlist, NULL);
	free(person_name);

	if (s_info.contact_id <= 0)
		return EINA_FALSE;

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Edit", _edit_button_clicked_cb, NULL);
	elm_toolbar_item_append(toolbar, NULL, "Delete", _delete_button_clicked_cb, NULL);

	data_contact_details_is_favourite(s_info.person_id, &is_favourite);
	fav_bt_caption = is_favourite ? CAPTION_BT_FAVOURITE_PRESSED : CAPTION_BT_FAVOURITE;
	view_create_button(s_info.navi, fav_bt_caption, "title_right_btn", "naviframe/title_right", _favourite_btn_clicked_cb, NULL);

	ret = contacts_db_add_changed_cb(_contacts_person._uri, _update_details_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed. Err= %d.", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on 'Edit' button click.
 * This function switches to the contact edit view.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _edit_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_contact_edit_create(s_info.navi, s_info.contact_id);
}

/**
 * @brief Internal callback function invoked on 'Delete' button click.
 * This function displays the popup window with requested operation confirmation prompt.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _delete_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	s_info.popup = view_create_popup(s_info.navi, "Warning", "Do you really want to delete?", _popup_ok_button_clicked_cb);
}

/**
 * @brief Internal callback function invoked on 'OK' button click from the popup window.
 * This function deletes currently viewed contact.
 * @param[in] data The user data passed to the view_create_popup() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _popup_ok_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	int ret;

	if (s_info.contact_id > 0) {
		ret = contacts_db_delete_record(_contacts_contact._uri, s_info.contact_id);
		if (ret != CONTACTS_ERROR_NONE)
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_delete_record() failed. Err= %d.", ret);
	}

	evas_object_del(s_info.popup);
	s_info.popup = NULL;

	elm_naviframe_item_pop(s_info.navi);
}

/**
 * @brief Internal callback function invoked on 'Fav' button click.
 * This function adds/removes the currently viewed contact to/from
 * the favorites contacts list.
 * @param[in] data The user data passed to the view_create_button() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _favourite_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	bool is_favourite = false;
	const char *fav_bt_caption = NULL;

	data_contact_details_toggle_favourites(s_info.person_id);
	data_contact_details_is_favourite(s_info.person_id, &is_favourite);

	fav_bt_caption = is_favourite ? CAPTION_BT_FAVOURITE_PRESSED : CAPTION_BT_FAVOURITE;
	elm_object_text_set(obj, fav_bt_caption);
}

/**
 * @brief Internal callback function invoked on genlist item display.
 * This function is assigned to the func.text_get handler of genlist item's class structure.
 * The item's area consists if two labels:
 * - top label where the contact item's description is displayed,
 * - bottom label where the contact item's value is displayed.
 * @param[in] data The user data passed to the elm_genlist_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] part The name of the item's part being rendered.
 * @return The text string to be displayed is returned.
 */
static char *_get_item_text_cb(void *data, Evas_Object *obj, const char *part)
{
	char *value = NULL;
	char *contact_value = NULL;

	if (!strcmp(part, "elm.text")) {
		const char *name = data_common_get_field_name((sc_common_contact_fields)data);
		if (name)
			return strdup(name);
	} else if (!strcmp(part, "elm.text.multiline")) {
			if (data_contact_details_get_value((sc_common_contact_fields)data, s_info.contact_id, &value))
				if (value) {
					contact_value = strdup(value);
					return contact_value;
				}
	}

	return NULL;
}

/**
 * @brief Internal callback function invoked on contact update.
 * This function displays all the items again.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _update_details_cb(const char *view_uri, void *user_data)
{
	char *person_name = NULL;

	if (s_info.person_id <= 0)
		return;

	if (!_get_contact_name(s_info.person_id, &person_name)) {
		view_common_return_to_view(s_info.navi, VI_PERSON_LIST);
		return;
	}

	elm_object_item_text_set(s_info.navi_item, person_name);
	free(person_name);

	_populate_list();
}

/**
 * @brief Internal function which obtains the contact's ID for given person ID.
 * @param[in] person_id ID of person.
 * @param[out] contact_id The contact ID for given person ID.
 * @return This function returns 'true' if the contact's ID was obtained successfully,
 * otherwise 'false' is returned.
 */
static bool _get_contact_id(int person_id, int *contact_id)
{
	int ret;
	contacts_record_h rec_h = NULL;

	ret = contacts_db_get_record(_contacts_person._uri, person_id, &rec_h);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err= %d.", ret);
		return false;
	}

	ret = contacts_record_get_int(rec_h, _contacts_person.display_contact_id, contact_id);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err= %d.", ret);

	contacts_record_destroy(rec_h, false);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Internal function which obtains the name for given ID.
 * @param[in] person_id ID of person.
 * @param[out] name The name for given person ID.
 * @return This function returns 'true' if the name was obtained successfully,
 * otherwise 'false' is returned.
 */
static bool _get_contact_name(int person_id, char **name)
{
	int ret;
	contacts_record_h rec_h = NULL;

	ret = contacts_db_get_record(_contacts_person._uri, person_id, &rec_h);
	if (ret != CONTACTS_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_record() failed. Err= %d.", ret);
		return false;
	}

	ret = contacts_record_get_str(rec_h, _contacts_person.display_name, name);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str() failed. Err= %d.", ret);

	contacts_record_destroy(rec_h, false);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Internal function which populates the selected contact list with
 * relevant detailed information.
 */
static void _populate_list(void)
{
	int i;
	Elm_Object_Item *item;
	Elm_Genlist_Item_Class *itc = create_genlist_item_class("multiline", _get_item_text_cb, NULL);

	elm_genlist_clear(s_info.genlist);

	for (i = (int)CF_PHONE_NUMBER; i < CF_MAX; i++) {
		item = elm_genlist_item_append(s_info.genlist, itc, (void *)i, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		elm_genlist_item_select_mode_set(item, ELM_OBJECT_SELECT_MODE_NONE);
	}

	elm_genlist_item_class_free(itc);
}
