/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <contacts.h>
#include <Elementary.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_contacts_list.h"
#include "view/view_contact_details.h"
#include "view/view_contact_edit.h"
#include "view/view_favorites_list.h"
#include "view/view_group_list.h"
#include "data/data_common.h"

static struct view_info {
	Evas_Object *navi;
	Evas_Object *list;
	Evas_Object *menu;
} s_info = {
	.navi = NULL,
	.list = NULL,
	.menu = NULL,
};

static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _update_list_cb(const char *view_uri, void *user_data);
static void _selected_list_cb(void *data, Evas_Object *obj, void *event_info);
static void _menu_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _categories_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _favourites_click_cb(void *data, Evas_Object *obj, void *event_info);
static void _create_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static void _populate_list(void);
static bool _create_context_menu(void);

/**
 * @brief Add person list.
 * @param[in] parent Parent object.
 * @return This function returns 'EINA_TRUE' if the person list view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_contacts_list_create(Evas_Object *parent)
{
	int ret;
	Evas_Object *toolbar;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid parent object for contacts list view.");
		return EINA_FALSE;
	}

	s_info.navi = parent;
	s_info.list = view_create_list(s_info.navi);
	if (!s_info.list)
		return EINA_FALSE;

	evas_object_event_callback_add(s_info.list, EVAS_CALLBACK_FREE, _destroy_list_cb, NULL);
	elm_naviframe_item_push(s_info.navi, "Contacts", NULL, NULL, s_info.list, NULL);

	_populate_list();

	if (!view_create_button(s_info.navi, "More", "toolbar_more_btn", NULL, _menu_button_clicked_cb, NULL))
		return EINA_FALSE;

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Create", _create_button_clicked_cb, NULL);

	ret = contacts_db_add_changed_cb(_contacts_person._uri, _update_list_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed. Err= %d.", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on contacts list destroy.
 * This function removes the 'changed' callback attached at the view creation
 * and deletes the context menu.
 * @param[in] data The user data passed to the contacts_db_add_changed_cb() function.
 * @param[in] e Evas surface of the object.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	contacts_db_remove_changed_cb(_contacts_person._uri, _update_list_cb, NULL);
	evas_object_del(s_info.menu);
	s_info.menu = NULL;
}

/**
 * @brief Internal callback function invoked on contacts list update.
 * This function removes all items from the list and populates it again.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _update_list_cb(const char *view_uri, void *user_data)
{
	elm_list_clear(s_info.list);
	_populate_list();
}

/**
 * @brief Internal callback function invoked on contacts list item selection.
 * This function displays the selected contact details.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _selected_list_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_list_item_selected_set(event_info, EINA_FALSE);

	view_contact_details_create(s_info.navi, (int)data);
}

/**
 * @brief Internal callback function invoked on HW Menu button click.
 * This function displays the context menu.
 * @param[in] data The user data passed to the contacts_db_add_changed_cb() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _menu_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	if (evas_object_visible_get(s_info.menu))
		elm_ctxpopup_dismiss(s_info.menu);
	else
		_create_context_menu();
}

/**
 * @brief Internal callback function invoked on 'Categories' item click from the
 * context menu.
 * This function displays the categories view.
 * @param[in] data The user data passed to the elm_ctxpopup_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _categories_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_ctxpopup_dismiss(s_info.menu);
	view_group_list_create(s_info.navi);
}

/**
 * @brief Internal callback function invoked on 'Favorites' item click from the
 * context menu.
 * This function displays the favorites view.
 * @param[in] data The user data passed to the elm_ctxpopup_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _favourites_click_cb(void *data, Evas_Object *obj, void *event_info)
{
	elm_ctxpopup_dismiss(s_info.menu);
	view_favourites_list_create(s_info.navi);
}

/**
 * @brief Internal callback function invoked on 'Create' button click.
 * This function switches to the contact creation view.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _create_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_contact_edit_create(s_info.navi, 0);
}

/**
 * @brief Internal function which adds contacts to the list.
 */
static void _populate_list(void)
{
	contacts_list_h person_list = NULL;
	contacts_record_h record = NULL;
	char *name = NULL;
	int id = 0;
	int ret;

	if (!data_common_get_all_records(_contacts_person._uri, &person_list))
		return;

	CONTACTS_LIST_FOREACH(person_list, record) {
		ret = contacts_record_get_str_p(record, _contacts_person.display_name, &name);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed. Err = %d", ret);
			continue;
		}

		if (!data_common_get_int(record, _contacts_person.id, &id))
			continue;

		elm_list_item_append(s_info.list, name, NULL, NULL, _selected_list_cb, (void *)id);
	}

	contacts_list_destroy(person_list, true);
	elm_list_go(s_info.list);
}

/**
 * @brief Internal function which creates and displays a context menu popup.
 * @return This function returns 'true' if the context menu object was created successfully,
 * otherwise 'false' is returned.
 */
static bool _create_context_menu(void)
{
	if (s_info.menu) {
		view_display_context_menu(s_info.menu, s_info.list);
		return true;
	}

	s_info.menu = view_create_context_menu(s_info.navi);
	if (!s_info.menu)
		return false;

	elm_ctxpopup_item_append(s_info.menu, "Categories", NULL, _categories_click_cb, NULL);
	elm_ctxpopup_item_append(s_info.menu, "Favorites", NULL, _favourites_click_cb, NULL);

	view_display_context_menu(s_info.menu, s_info.list);

	return true;
}
