/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <contacts.h>
#include "contacts_app.h"
#include "view.h"
#include "view/view_group_list.h"
#include "view/view_group_create.h"
#include "view/view_group_details.h"
#include "data/data_common.h"

static struct view_info {
	Evas_Object *navi;
	Elm_Object_Item *navi_item;
	Evas_Object *list;
	contacts_list_h group_list;
} s_info = {
	.navi = NULL,
	.navi_item = NULL,
	.list = NULL,
	.group_list = NULL,
};

static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _list_changed_cb(const char *view_uri, void *user_data);
static void _item_selected_cb(void *data, Evas_Object *obj, void *item);
static void _create_button_clicked_cb(void *data, Evas_Object *obj, void *event_info);
static bool _load_groups(void);
static void _populate_list(void);

/**
 * @brief Adds group list view.
 * @param[in] parent Parent object.
 * @return This function returns 'EINA_TRUE' if the view was created successfully,
 * otherwise 'EINA_FALSE' is returned.
 */
Eina_Bool view_group_list_create(Evas_Object *parent)
{
	int ret;
	Evas_Object *toolbar;

	if (!parent) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Invalid input argument for group list view.");
		return EINA_FALSE;
	}

	s_info.navi = parent;
	s_info.list = view_create_list(s_info.navi);
	if (!s_info.list)
		return EINA_FALSE;

	evas_object_event_callback_add(s_info.list, EVAS_CALLBACK_FREE, _destroy_list_cb, NULL);
	s_info.navi_item = elm_naviframe_item_push(s_info.navi, "Categories", NULL, NULL, s_info.list, NULL);

	_load_groups();
	_populate_list();

	toolbar = view_create_toolbar(s_info.navi, "toolbar");
	if (!toolbar)
		return EINA_FALSE;

	elm_toolbar_item_append(toolbar, NULL, "Create", _create_button_clicked_cb, NULL);

	ret = contacts_db_add_changed_cb(_contacts_group._uri, _list_changed_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed for group. Err= %d.", ret);

	ret = contacts_db_add_changed_cb(_contacts_group_relation._uri, _list_changed_cb, NULL);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_add_changed_cb() failed for group relation. Err= %d.", ret);

	return EINA_TRUE;
}

/**
 * @brief Internal callback function invoked on group list destroy.
 * This function removes the 'changed' callbacks attached at the view creation
 * and deletes the group list.
 * @param[in] data The user data passed to the contacts_db_add_changed_cb() function.
 * @param[in] e Evas surface of the object.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _destroy_list_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	contacts_db_remove_changed_cb(_contacts_group._uri, _list_changed_cb, NULL);
	contacts_db_remove_changed_cb(_contacts_group_relation._uri, _list_changed_cb, NULL);
	contacts_list_destroy(s_info.group_list, true);
}

/**
 * @brief Internal callback function invoked on group list update.
 * This function removes all items from the list and populates it again.
 * @param[in] view_uri The URI of the view.
 * @param[in] user_data The data passed to the callback attachment function.
 */
static void _list_changed_cb(const char *view_uri, void *user_data)
{
	contacts_list_destroy(s_info.group_list, true);

	_load_groups();

	elm_list_clear(s_info.list);
	_populate_list();
}

/**
 * @brief Internal callback function invoked on group list item selection.
 * This function displays the selected group contacts.
 * @param[in] data The user data passed to the elm_list_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _item_selected_cb(void *data, Evas_Object *obj, void *item)
{
	elm_list_item_selected_set(item, EINA_FALSE);

	view_group_details_create(s_info.navi, (int)data);
}

/**
 * @brief Internal callback function invoked on 'Create' button click.
 * This function switches to the group creation view.
 * @param[in] data The user data passed to the elm_toolbar_item_append() function.
 * @param[in] obj The object invoking this callback function.
 * @param[in] event_info The structure containing the information on this event.
 */
static void _create_button_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	view_group_create_create(s_info.navi);
}

/**
 * @brief Internal function which loads all the available groups into a list.
 * @return This function returns 'true' if the groups were loaded successfully,
 * otherwise 'false' is returned.
 */
static bool _load_groups(void)
{
	int ret = contacts_db_get_all_records(_contacts_group._uri, 0, 0, &s_info.group_list);
	if (ret != CONTACTS_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "contacts_db_get_all_records() failed. Err = %d.", ret);

	return (ret == CONTACTS_ERROR_NONE);
}

/**
 * @brief Internal function which adds groups to the list.
 */
static void _populate_list(void)
{
	char *group_name;
	int group_id;
	int count;
	int ret;
	char label[256] = {0,};
	contacts_record_h group;
	contacts_list_h members;

	CONTACTS_LIST_FOREACH(s_info.group_list, group) {
		ret = contacts_record_get_str_p(group, _contacts_group.name, &group_name);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_str_p() failed. Err = %d.", ret);
			continue;
		}

		ret = contacts_record_get_int(group, _contacts_group.id, &group_id);
		if (ret != CONTACTS_ERROR_NONE) {
			dlog_print(DLOG_ERROR, LOG_TAG, "contacts_record_get_int() failed. Err = %d.", ret);
			continue;
		}

		members = data_common_get_int_records(_contacts_group_relation._uri, _contacts_group_relation.group_id, group_id);
		if (!members)
			continue;

		contacts_list_get_count(members, &count);
		contacts_list_destroy(members, true);

		snprintf(label, sizeof(label), "%s (%d)", group_name, count);
		elm_list_item_append(s_info.list, label, NULL, NULL, _item_selected_cb, (void *)group_id);
	}

	elm_list_go(s_info.list);
}
